require 'fog/google'

require 'linecook-gem/packager/packer/base'

module Linecook
  class GCPPacker < Linecook::Packer

    BUILDER_CONFIG = {
      type: 'null',
    }.freeze

    # run with 'shell-local'
    PRE_MOUNT_COMMANDS = [

      # create a sparse image with dd
      "dd if=/dev/zero of=#{image_path} bs=1 count=0 seek=#{image_size}",
      "parted -s $(losetup | grep #{image_path} | awk '{print $1}') mklabel msdos",
      "parted -s $(losetup | grep #{image_path} | awk '{print $1}') mkpart primary ext4 0% 100%",
      "mkfs.ext4 $(losetup | grep #{image_path} | awk '{print $1}')p1",
      "tune2fs -L cloudimg-rootfs $(losetup | grep #{image_path} | awk '{print $1}')p1",
      "mount $(losetup | grep #{image_path} | awk '{print $1}')p1 /mnt/#{build_uuid}"
    ]

    CHROOT_SETUP = [
#      tar -C "${MOUNT_PATH}" -xpf "${TAR_PATH}"
#
#mount --bind /dev "${MOUNT_PATH}"/dev
#mount --bind /proc "${MOUNT_PATH}"/proc
#mount --bind /sys "${MOUNT_PATH}"/sys
#
#echo "LABEL=cloudimg-rootfs   /        ext4   defaults,discard        0 0" >> "${MOUNT_PATH}"/etc/fstab
    ]

    CHROOT_COMMNANDS_EXTRA = [
#APT_PACKAGES=(
#  grub-pc
#  cloud-init
#  cloud-initramfs-growroot
#  gce-daemon
#  gce-startup-scripts
#  gce-image-bundle
#  gce-cloud-config
#)
#
#DEBIAN_FRONTEND=noninteractive apt-get -o Dpkg::Options::="--force-confdef" \
#  -o Dpkg::Options::="--force-confold" -y --force-yes --no-upgrade install "${APT_PACKAGES}"
#
#rm -f /etc/hostname
#ln -s /usr/share/google/set-hostname /etc/dhcp/dhclient-exit-hooks.d/
#
#echo "169.254.169.254 metadata.google.internal metadata" >> /etc/hosts
#echo "127.0.0.1 localhost" >> /etc/hosts
    ]

   CHROOT_UNMOUNT = [
     "umount /mnt/#{build_uuid}/dev",
     "umount /mnt/#{build_uuid}/sys",
     "umount /mnt/#{build_uuid}/proc",
     "umount /mnt/#{build_uuid}"
   ]

   COMPRESS_IMAGE = [
     "losetup -d $(losetup | grep #{image_path} | awk '{print $1}')",
     "mkdir -p /tmp/#{build_uuid}",
     "mv #{image_path} /tmp/#{build_uuid}/disk.raw", #must be called disk.raw
     "cd /tmp/#{build_uuid} && tar -czvf #{build_uuid}.tar.gz disk.raw",
     "rm /tmp#{build_uuid}/disk.raw"
   ]


  private

    def upload
      connection = Fog::Storage::Google.new

      image_source = File.open("/tmp/#{build_uuid}/#{build_uuid}.tar.gz")
      connection.put_object("bucket", "#{image_name}-#{build_uuid}.tar.gz", image_source)
      image_source.close
      image_source.unlink

    end

    def create_gcp_image
      connection = Fog::Compute.new(:provider => "Google")

      rawdisk = {
        source: "gs://#{bucket}/#{image_slug}",
        container_type: "TAR"
      }


      img = connection.images.create(name: "test-image", # FIXME - from config
                                     description: "Test image (via fog)", # FIXME - from config
                                     raw_disk: rawdisk)

      img.reload # will raise if image was not saved correctly
    end

    # run with 'shell-local' and "execute_command"
    #  https://www.packer.io/docs/provisioners/shell-local.html
    # to wrap everything with chroot

# Rather than use the 'googlecloud' provider in packer, which would require us to spin up a VM
# We post-process the image locally, and upload it to google cloud. Google is able to deal with tarballs
# directly, so there's no need for a fancy chroot builder.
#
# Gist of how it works:
# 1. Use a null builder to post-process a linecook image into an acceptable format for google
# 2. Snapshot that image as a tarball
# 3. Upload that image to google storage
# 4. Create a google compute image from the tarball URL.  https://cloud.google.com/compute/docs/images/import-existing-image


# https://github.com/Shopify/google-cloud-images/blob/master/deps/chroot-commands.sh
# https://github.com/Shopify/google-cloud-images/blob/master/deps/generate-image.sh

  end
end
